//+------------------------------------------------------------------+
//|                                                      MAAngle.mq4 |
//|                                               original    jpkfox |
//|                                               edited by dariuske |
//| You can use this indicator to measure when the MA angle is       |
//| "near zero". AngleTreshold determines when the angle for the     |
//| EMA is "about zero": This is when the value is between           |
//| [-AngleTreshold, AngleTreshold] (or when the histogram is red).  |
//|   MAMode : 0 = SMA, 1 = EMA, 2 = Smoothed, 3 = Weighted          |
//|   MAPeriod: MA period                                            |
//|   AngleTreshold: The angle value is "about zero" when it is      |
//|     between the values [-AngleTreshold, AngleTreshold].          |      
//|   StartMAShift: The starting point to calculate the              |   
//|     angle. This is a shift value to the left from the            |
//|     observation point. Should be StartEMAShift > EndEMAShift.    | 
//|   StartMAShift: The ending point to calculate the                |
//|     angle. This is a shift value to the left from the            | 
//|     observation point. Should be StartEMAShift > EndEMAShift.    |
//|                                                                  |
//|   Modified by MrPip                                              |
//|       Red for down                                               |
//|       Yellow for near zero                                       |
//|       Green for up                                               |
//|  10/15/05  MrPip                                                 |
//|            Corrected problem with USDJPY and optimized code      |
//|  10/23/05  Added other JPY crosses                               |
//|                                                                  |
//|                                                                  |  
//|                                                                  |  
//|  12/01/07  Dariuske: Changed code for SMA50 (PhilNelSystem)      |
//|  18/01/07  Dariuske: Changed code for multiple mode MA's         |
//+------------------------------------------------------------------+

#property  copyright "jpkfox"
#property  link      "http://www.strategybuilderfx.com/forums/showthread.php?t=15274&page=1&pp=8"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  LimeGreen
#property indicator_color2  FireBrick 
#property indicator_color3  Yellow
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2

//
//
//
//
//


extern int    MAPeriod        = 34;
extern string m               = "--Moving Average Types--";
extern string m0              = " 0 = SMA";
extern string m1              = " 1 = EMA";
extern string m2              = " 2 = SMMA";
extern string m3              = " 3 = LWMA";
extern string m4              = " 4 = LSMA";
extern int    MAType          = 1;
extern string p               = "--Price Types--";
extern string p0              = " 0 = close";
extern string p1              = " 1 = open";
extern string p2              = " 2 = high";
extern string p3              = " 3 = low";
extern string p4              = " 4 = median(high+low)/2";
extern string p5              = " 5 = typical(high+low+close)/3";
extern string p6              = " 6 = weighted(high+low+close+close)/4";
extern int    MA_AppliedPrice = 4;
extern double AngleTreshold   = 15;
extern int    PreMAShift      = 4;
extern int    CurMAShift      = 0;

extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = false;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsNotify    = false;
extern bool   alertsEmail     = false;
extern string soundfile       = "alert2.wav"; 

//
//
//
//
//

double up[];
double dn[];
double zl[];
double trend[];

int    MAMode;
string strMAType;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,up); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,dn); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,zl); SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,trend);

    switch (MAType)
   {
      case 1:  strMAType = " EMA ";  MAMode = MODE_EMA;  break;
      case 2:  strMAType = " SMMA "; MAMode = MODE_SMMA; break;
      case 3:  strMAType = " LWMA "; MAMode = MODE_LWMA; break;
      case 4:  strMAType = " LSMA ";                     break;
      default: strMAType = " SMA ";  MAMode = MODE_SMA;  break;
   }
   IndicatorShortName(strMAType+ "Angle ("+MAPeriod+","+AngleTreshold+","+PreMAShift+","+CurMAShift+")");
   return(0);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int start()
{
   if(CurMAShift >= PreMAShift)
   {
      Print("Error: CurMAShift >= PrevMAShift");
      PreMAShift = 6;
      CurMAShift = 0;      
   }  
   int counted_bars=IndicatorCounted();
   int i,limit;
   
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-1); 
   
   //
   //
   //
   //
   //
   
   double dFactor = 2*3.14159/180.0;
   double mFactor = 100000.0;
   string Sym     = StringSubstr(Symbol(),3,3); if (Sym == "JPY") mFactor = 1000.0;
   int ShiftDif   = PreMAShift-CurMAShift;
   mFactor       /= ShiftDif; 
   for(i=limit; i>=0; i--)
   {
      if (MAType == 4)
      {
         double fCurMA = 3.0*iMA(NULL,0,MAPeriod,0,MODE_LWMA,MA_AppliedPrice,i+CurMAShift)-2.0*iMA(NULL,0,MAPeriod,0,MODE_SMA,MA_AppliedPrice,i+CurMAShift);
         double fPreMA = 3.0*iMA(NULL,0,MAPeriod,0,MODE_LWMA,MA_AppliedPrice,i+PreMAShift)-2.0*iMA(NULL,0,MAPeriod,0,MODE_SMA,MA_AppliedPrice,i+PreMAShift);
      }
      else
      {
         fCurMA =iMA(NULL,0,MAPeriod,0,MAMode,MA_AppliedPrice,i+CurMAShift);
         fPreMA =iMA(NULL,0,MAPeriod,0,MAMode,MA_AppliedPrice,i+PreMAShift);
      } 
      
      double fAngle = mFactor * (fCurMA - fPreMA)*0.5;
      up[i] = EMPTY_VALUE;
      dn[i] = EMPTY_VALUE;
      zl[i] = EMPTY_VALUE;
      trend[i] = 0;
      
      if (fAngle> AngleTreshold)                              trend[i] = 1;
      if (fAngle<-AngleTreshold)                              trend[i] =-1;
      if (fAngle <= AngleTreshold && fAngle >= AngleTreshold) trend[i] = 0;  
      if (trend[i] == 1) up[i] = fAngle;
      if (trend[i] ==-1) dn[i] = fAngle;
      if (trend[i] == 0) zl[i] = fAngle;
   }
manageAlerts();   
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; 
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1)                          doAlert(whichBar,"up");
         if (trend[whichBar] ==-1)                          doAlert(whichBar,"down");
         if (trend[whichBar] == 0 && trend[whichBar+1]== 1) doAlert(whichBar,"back from up now neutral");
         if (trend[whichBar] ==-0 && trend[whichBar+1]==-1) doAlert(whichBar,"back from down now neutral");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //`
          //
          //
          //
          //

          message =  StringConcatenate(Symbol(),TimeToStr(TimeLocal(),TIME_SECONDS), strMAType+" Angle ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(), strMAType+" Angle "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}


